
/*
 * Dummy placeholder to make the EFI patches apply to the x86 tree.
 * Andrew/Len, please just kill this file if you encounter it.
 */
#ifndef acpi_reboot
# define acpi_reboot() do { } while (0)
#endif

